
#include <Arduino_AdvancedAnalog.h>
#include <arm_math.h> // CMSIS DSP
#define BUFSIZE 32
#define NBUF 10
#define NSAMPLES 4096
AdvancedADC adc1(A0);
float sig[NSAMPLES];
uint32_t sample_rate = 40000;

// transformation nombre -> tension
float offset = 2048;
float num2volt = 3.217/4096;

// paramètres de détection des raies
float Amin = 0.01;
float epsilon = 1e-2;

uint16_t indice;
arm_rfft_fast_instance_f32 rfft_instance;
float fft[NSAMPLES*2];
float fft_amp[NSAMPLES];
float deltaf;
float Veff = 0;

void setup() {
  Serial.begin(115200);
  while(!Serial);
  indice = 0;
  deltaf = sample_rate*1.0/NSAMPLES;
  if (!adc1.begin(AN_RESOLUTION_12, sample_rate, BUFSIZE, NBUF)) {
        while (1);
  }
}

void printRaie(float f,float amp) {
    Serial.print(f,6);
    Serial.print(" +/- ");
    Serial.print(deltaf,4);
    Serial.print(" Hz, ");
    Serial.println(amp,8);
}

void interpolRaie(int i) {
   #define a fft_amp[i-1]
   #define b fft_amp[i]
   #define c fft_amp[i+1]
   float p = 0.5*(a-c)/(1-2*b+c);
   float f = (i+p)*deltaf;
   float A = b-0.25*(a-c)*p;
   printRaie(f,A);
}

void loop() {
  
  if (adc1.available()) {
      SampleBuffer buf1 = adc1.read();
      uint16_t *buf1_data = buf1.data();
      for (uint16_t i=0; i<BUFSIZE; i++) {
        sig[indice] = (buf1_data[i]-offset)*num2volt;
        Veff += sig[indice]*sig[indice];
        indice++;
      }
      buf1.release();
      if (indice==NSAMPLES) {
        indice = 0;
        arm_rfft_fast_init_f32(&rfft_instance,NSAMPLES);
        arm_rfft_fast_f32(&rfft_instance,sig,fft,0);
        arm_cmplx_mag_f32(fft,fft_amp,NSAMPLES/2);
        float f;
        uint16_t N = NSAMPLES/2;
        uint16_t i;
        for (i=0; i<N; i++) {
            fft_amp[i] *= 2.0/NSAMPLES;
            
        }
        
        Serial.println("---------------");
        Veff = sqrt(Veff/NSAMPLES);
        Serial.print("Veff = "); Serial.println(Veff,5);
        float deriv;
        int8_t signe;
        i = 1;
        while (i<N-1) {
            
            if (fft_amp[i]>Amin) {
                deriv = fft_amp[i]-fft_amp[i-1];
                if (deriv > 0) signe=1; else signe = -1;
                deriv = fft_amp[i+1]-fft_amp[i];
                if ((signe>0)&&(deriv<-epsilon)) {
                  signe = -1;
                  interpolRaie(i);
                  i++;
                }
                while ((i<N-1)&&(fft_amp[i]>Amin)) {
                  deriv = fft_amp[i]-fft_amp[i-1];
                  if ((signe>0)&&(deriv<-epsilon)) {
                    signe = -1;
                    interpolRaie(i-1);
                  }
                  else if ((signe<0)&&(deriv>epsilon)) signe = 1;
                  i++;
                }
            }
            else i++;

        }
        delay(1000);
      }
  }

}

                  
#include <Arduino_AdvancedAnalog.h>

#define BUFSIZE 32
#define NBUF 10
#define NSAMPLES 16384
#define P 14 // NSAMPLE = 2**P
AdvancedADC adc1(A0);
float A_re[NSAMPLES];
float A_im[NSAMPLES];
float B_re[NSAMPLES];
float B_im[NSAMPLES];
uint32_t sample_rate = 50000;

// transformation nombre -> tension
float offset = 2048;
float num2volt = 3.217/4096;

uint16_t indice;
float fft_amp[NSAMPLES/2];
float deltaf;
float Amin = 0.001;
float epsilon = 1e-2;
float Veff = 0;

int inversion(uint16_t i) {
  uint8_t bi;
  int8_t b;
  uint8_t bits[P];
  for (b=0; b<P; b++) {
    bi = i & 0b1;
    i = i >> 1;
    bits[b] = bi;
  }
  uint16_t facteur = 1;
  uint16_t j = 0;
  for (b=P-1; b>=0; b--) {
    j += bits[b]*facteur;
    facteur *= 2;
  }
  return j;
}        


void fft() {
  uint16_t j;
  for (int k=0; k<NSAMPLES; k++) {
      j = inversion(k);
      B_re[j] = A_re[k];
      B_im[k] = A_im[k];
  }
  uint16_t taille = 1;
  uint16_t taille_precedente;
  uint16_t nombre_tfd = NSAMPLES;
  uint16_t position;
  float phi, W_re, W_im;;
  float x,y;
  uint8_t parite = 0;
  for (uint16_t q=0; q<P; q++) {
      taille_precedente = taille;
      taille *= 2;
      nombre_tfd /= 2;
      for (uint16_t m=0; m<nombre_tfd; m++) {
          phi = -2*PI/taille;
          position = m*taille;
          uint16_t i,j,k;
          for (i=0; i<taille_precedente; i++) {
              W_re = cos(phi*i);
              W_im = sin(phi*i);
              j = position+taille_precedente+i;
              k = position +i;
              if (parite==0) {
                x = B_re[j];
                y = B_im[j];
                A_re[k] = B_re[k] + x*W_re-y*W_im;
                A_im[k] = B_im[k] + x*W_im+y*W_re;
              }
              else {
                x = A_re[j];
                y = A_im[j];
                B_re[k] = A_re[k] + x*W_re-y*W_im;
                B_im[k] = A_im[k] + x*W_im+y*W_re;
              }
          }
          for (i=taille_precedente; i<taille; i++) {
              W_re = cos(phi*i);
              W_im = sin(phi*i);
              j = position+i;
              k = position+i-taille_precedente;
              if (parite==0) {
                x = B_re[j];
                y = B_im[j];
                A_re[j] = B_re[k] + x*W_re-y*W_im;
                A_im[j] = B_im[k] + x*W_im+y*W_re;
              }
              else {
                x = A_re[j];
                y = A_im[j];
                B_re[j] = A_re[k] + x*W_re-y*W_im;
                B_im[j] = A_im[k] + x*W_im+y*W_re;
              }   
          }  
      }
      parite = ~parite;
  }
  if (parite) {
      for (int k=0; k<NSAMPLES; k++) {B_re[k] = A_re[k]; B_im[k] = A_im[k];}
  }
}                 


void setup() {
  Serial.begin(115200);
  while(!Serial);
  indice = 0;
  deltaf = sample_rate*1.0/NSAMPLES;
  for (int i=0; i<NSAMPLES; i++) A_im[i] = 0.0;
  if (!adc1.begin(AN_RESOLUTION_12, sample_rate, BUFSIZE, NBUF)) {
        while (1);
  }

}

void printRaie(float f,float amp) {
    Serial.print(f,6);
    Serial.print(" +/- ");
    Serial.print(deltaf,4);
    Serial.print(" Hz, ");
    Serial.println(amp,8);
}

void interpolRaie(int i) {
   #define a fft_amp[i-1]
   #define b fft_amp[i]
   #define c fft_amp[i+1]
   float p = 0.5*(a-c)/(1-2*b+c);
   float f = (i+p)*deltaf;
   float A = b-0.25*(a-c)*p;
   printRaie(f,A);
}


void loop() {
  
  if (adc1.available()) {
      
      SampleBuffer buf1 = adc1.read();
      uint16_t *buf1_data = buf1.data();
      
      for (uint16_t i=0; i<BUFSIZE; i++) {
        A_re[indice] = (buf1_data[i]-offset)*num2volt;
        A_im[indice] = 0.0;
        Veff += A_re[indice]*A_re[indice];
        indice++;
      }
      buf1.release();
      if (indice==NSAMPLES) {
        indice = 0;
        Veff = sqrt(Veff/NSAMPLES);
        Serial.println("---------------");
        Serial.print("Veff = "); Serial.println(Veff,5);
        Veff = 0;
        fft();
        float f;
        uint16_t N = NSAMPLES/2;
        uint16_t i;
        for (i=0; i<N; i++) {
            fft_amp[i] = sqrt(B_re[i]*B_re[i]+B_im[i]*B_im[i])*2.0/NSAMPLES;
            
        }
        
        float deriv;
        int8_t signe;
        i = 1;
        while (i<N-1) {
            
            if (fft_amp[i]>Amin) {
                deriv = fft_amp[i]-fft_amp[i-1];
                if (deriv > 0) signe=1; else signe = -1;
                deriv = fft_amp[i+1]-fft_amp[i];
                if ((signe>0)&&(deriv<-epsilon)) {
                  signe = -1;
                  interpolRaie(i);
                  i++;
                }
                while ((i<N-1)&&(fft_amp[i]>Amin)) {
                  deriv = fft_amp[i]-fft_amp[i-1];
                  if ((signe>0)&&(deriv<-epsilon)) {
                    signe = -1;
                    interpolRaie(i-1);
                  }
                  else if ((signe<0)&&(deriv>epsilon)) signe = 1;
                  i++;
                }
            }
            else i++;

        }
        delay(1000);
      }
  }

}

                         